//----------------------------------------------------------------------------
// Copyright (c) 2007-2008 Fabrice HARROUET (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// The author makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------------

#include "sniffer.h"

#include <stdlib.h>

void
displayUsage(const string & cmd)
{
string spc=string(cmd.size(),' ');
cerr <<
"usage: " << cmd << " -mac @MAC -to @IP [ -if name ] \\\n"
"       " << spc << " [ -from @IP ] ... \\\n"
"       " << spc << " [ -tcp @IP:serverPort:localPort] ... \\\n"
"       " << spc << " [ -udp @IP:serverPort:localPort] ...\n"
"\n"
"    -mac @MAC                                     (mandatory)\n"
"        an unused MAC address to spoof ARP\n"
"\n"
"    -to @IP                                       (mandatory)\n"
"        target of traffic to listen to\n"
"\n"
"    -if name                               (defaults to eth0)\n"
"        interface name to listen to\n"
"\n"
"    -from @IP                                     (0 or many)\n"
"        origin of traffic to listen to\n"
"\n"
"    -tcp @IP:serverPort:localPort                 (0 or many)\n"
"        redirect TCP traffic that should reach @IP:serverPort\n"
"        to a local TCP server listening on localPort\n"
"\n"
"    -udp @IP:serverPort:localPort                 (0 or many)\n"
"        redirect UDP traffic that should reach @IP:serverPort\n"
"        to a local UDP server listening on localPort\n"
<< endl;
}

int
main(int argc,
     char ** argv)
{
cerr << endl << "Fingers In the Middle Of the Nose ..." << endl;
cerr << "(man-in-the-middle with the fingers-in-the-nose)" << endl << endl;
string ifName("eth0");
uint64_t fakeMac=0;
uint32_t toIp=0;
vector<uint32_t> fromIp;
vector<Sniffer::Capture> tcpCapture;
vector<Sniffer::Capture> udpCapture;
bool showUsage=false;
for(int i=1;i<argc;i++)
  {
  if(!strcmp(argv[i],"-from"))
    {
    uint32_t from;
    if((++i>=argc)||!IpPacket::parseIpAddr(from,argv[i]))
      {
      showUsage=true;
      }
    else
      {
      fromIp.push_back(from);
      }
    }
  else if(!strcmp(argv[i],"-to"))
    {
    if((++i>=argc)||!IpPacket::parseIpAddr(toIp,argv[i]))
      {
      showUsage=true;
      }
    }
  else if(!strcmp(argv[i],"-mac"))
    {
    if((++i>=argc)||!EthFrame::parseMacAddr(fakeMac,argv[i]))
      {
      showUsage=true;
      }
    }
  else if(!strcmp(argv[i],"-if"))
    {
    if(++i>=argc)
      {
      showUsage=true;
      }
    else
      {
      ifName=argv[i];
      }
    }
  else if(!strcmp(argv[i],"-tcp")||!strcmp(argv[i],"-udp"))
    {
    if(++i>=argc)
      {
      showUsage=true;
      }
    else
      {
      bool ok=false;
      Sniffer::Capture capt;
      char * c1=strchr(argv[i],':');
      if(c1&&IpPacket::parseIpAddr(capt.serverIp,argv[i]))
        {
        char * c2=strchr(c1+1,':');
        if(c2&&(sscanf(c1+1,"%hu",&capt.serverPort)==1))
          {
          if(sscanf(c2+1,"%hu",&capt.localPort)==1)
            {
            ok=true;
            }
          }
        }
      if(ok)
        {
        if(!strcmp(argv[i-1],"-tcp"))
          {
          tcpCapture.push_back(capt);
          }
        else
          {
          udpCapture.push_back(capt);
          }
        }
      else
        {
        showUsage=true;
        }
      }
    }
  else
    {
    showUsage=true;
    }
  }
if(!fakeMac||!toIp||showUsage)
  {
  displayUsage(argv[0]);
  return EXIT_FAILURE;
  }
else
  {
  Sniffer sniffer(10.0,ifName,fakeMac,toIp,fromIp,tcpCapture,udpCapture);
  return sniffer.mainLoop() ? EXIT_SUCCESS : EXIT_FAILURE;
  }
}

//----------------------------------------------------------------------------
