//----------------------------------------------------------------------------
// Copyright (c) 2007-2008 Fabrice HARROUET (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// The author makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------------

#ifndef PACKETTYPES_H
#define PACKETTYPES_H 1

#define __STDC_LIMIT_MACROS 1
#include <inttypes.h>

#include <iostream>
#include <string>
using namespace std;

#include <stdio.h>
#include <ctype.h>

//----------------------------------------------------------------------------
// !!! Warning !!! All the code provided here is inline and _NOT_virtual_ !!!
//----------------------------------------------------------------------------

//-------- DataPacket --------------------------------------------------------

class DataPacket
{
public:

  inline
  DataPacket(size_t size);

  inline
  DataPacket(uint8_t * ptr,
             size_t size);

  inline
  ~DataPacket(void);

  inline
  void
  reinit(size_t size);

  inline
  void
  reinit(uint8_t * ptr,
         size_t size);

  inline
  const uint8_t *
  getPtr(void) const;

  inline
  uint8_t *
  accessPtr(void);

  inline
  size_t
  getSize(void) const;

  inline static
  uint8_t
  readUint4L(const uint8_t * ptr);

  inline static
  void
  writeUint4L(uint8_t * ptr,
              uint8_t value);

  inline static
  uint8_t
  readUint4H(const uint8_t * ptr);

  inline static
  void
  writeUint4H(uint8_t * ptr,
              uint8_t value);

  inline static
  uint8_t
  readUint8(const uint8_t * ptr);

  inline static
  void
  writeUint8(uint8_t * ptr,
             uint8_t value);

  inline static
  uint16_t
  readUint16(const uint8_t * ptr);

  inline static
  void
  writeUint16(uint8_t * ptr,
              uint16_t value);

  inline static
  uint32_t
  readUint32(const uint8_t * ptr);

  inline static
  void
  writeUint32(uint8_t * ptr,
              uint32_t value);

  inline static
  uint64_t
  readUint48(const uint8_t * ptr);

  inline static
  void
  writeUint48(uint8_t * ptr,
              uint64_t value);

  inline static
  uint64_t
  readUint64(const uint8_t * ptr);

  inline static
  void
  writeUint64(uint8_t * ptr,
              uint64_t value);

  inline static
  void
  showRaw(ostream & output,
          const uint8_t * ptr,
          size_t dataSize);

  inline static
  void
  showHexa(ostream & output,
           const uint8_t * ptr,
           size_t dataSize);

  inline static
  void
  showText(ostream & output,
           const uint8_t * ptr,
           size_t dataSize);

protected:
  uint8_t * _ptr;
  size_t _size;
  bool _owner;
private:
  DataPacket(void); // no implementation
  DataPacket(const DataPacket &); // no implementation
  DataPacket & operator=(const DataPacket &); // no implementation
};

//-------- EthFrame ----------------------------------------------------------

class EthFrame : public DataPacket
{
public:

  inline
  EthFrame(size_t size);

  inline
  EthFrame(uint8_t * ptr,
           size_t size);

  inline
  uint64_t
  readDstMac(void) const;

  inline
  void
  writeDstMac(uint64_t dstMac);

  inline
  uint64_t
  readSrcMac(void) const;

  inline
  void
  writeSrcMac(uint64_t srcMac);

  inline
  uint16_t
  readEthProto(void) const;

  inline
  void
  writeEthProto(uint16_t ethProto);

  inline
  void
  readEthHeader(uint64_t & dstMacOut,
                uint64_t & srcMacOut,
                uint16_t & ethProtoOut) const;

  inline
  void
  writeEthHeader(uint64_t dstMac,
                 uint64_t srcMac,
                 uint16_t ethProto);
                
  inline
  size_t
  getEthHeaderSkip(void) const;

  inline static
  bool
  parseMacAddr(uint64_t & macAddrOut,
               const char * textAddr);

  inline static
  void
  dumpMacAddr(char * textAddrOut,
              uint64_t macAddr);

  inline static
  string
  showMacAddr(uint64_t macAddr);
  
  inline
  void
  showEthHeader(ostream & output,
                size_t indent) const;

  enum
    {
    HEADER_SIZE=14,
    MIN_FRAME_SIZE=60,
    PROTO_IP=0x0800,
    PROTO_ARP=0x0806
    };

  static const uint64_t MAC_BROADCAST=0x0000FFFFFFFFFFFFLLU;

private:
  EthFrame(void); // no implementation
  EthFrame(const EthFrame &); // no implementation
  EthFrame & operator=(const EthFrame &); // no implementation
};

//-------- ArpPacket ---------------------------------------------------------

class ArpPacket : public DataPacket
{
public:

  inline
  ArpPacket(size_t size);

  inline
  ArpPacket(uint8_t * ptr,
            size_t size);

  inline
  ArpPacket(EthFrame & eth);

  inline
  uint16_t
  readHwType(void) const;

  inline
  void
  writeHwType(uint16_t hwType) const;

  inline
  uint16_t
  readProtoType(void) const;

  inline
  void
  writeProtoType(uint16_t protoType) const;

  inline
  uint8_t
  readHwSize(void) const;

  inline
  void
  writeHwSize(uint8_t hwSize) const;

  inline
  uint8_t
  readProtoSize(void) const;

  inline
  void
  writeProtoSize(uint8_t protoSize) const;

  inline
  uint16_t
  readOperation(void) const;

  inline
  void
  writeOperation(uint16_t operation) const;

  inline
  uint64_t
  readSrcHw(void) const;

  inline
  void
  writeSrcHw(uint64_t srcHw);

  inline
  uint32_t
  readSrcProto(void) const;

  inline
  void
  writeSrcProto(uint32_t srcProto);

  inline
  uint64_t
  readDstHw(void) const;

  inline
  void
  writeDstHw(uint64_t dstHw);

  inline
  uint32_t
  readDstProto(void) const;

  inline
  void
  writeDstProto(uint32_t dstProto);

  inline
  void
  readArpHeader(uint16_t & hwTypeOut,
                uint16_t & protoTypeOut,
                uint8_t & hwSizeOut,
                uint8_t & protoSizeOut,
                uint16_t & operationOut,
                uint64_t & srcHwOut,
                uint32_t & srcProtoOut,
                uint64_t & dstHwOut,
                uint32_t & dstProtoOut) const;
  
  inline
  void
  writeArpHeader(uint16_t hwType,
                 uint16_t protoType,
                 uint8_t hwSize,
                 uint8_t protoSize,
                 uint16_t operation,
                 uint64_t srcHw,
                 uint32_t srcProto,
                 uint64_t dstHw,
                 uint32_t dstProto);

  inline
  void
  showArpHeader(ostream & output,
                size_t indent) const;

  enum
    {
    HEADER_SIZE=28,
    HWTYPE_ETH=1,
    HWSIZE_ETH=6,
    PROTOTYPE_IP=0x0800,
    PROTOSIZE_IP=4,
    ARP_REQUEST=1,
    ARP_REPLY=2
    };

private:
  ArpPacket(void); // no implementation
  ArpPacket(const ArpPacket &); // no implementation
  ArpPacket & operator=(const ArpPacket &); // no implementation
};

//-------- IpPacket ----------------------------------------------------------

class IpPacket : public DataPacket
{
public:

  inline
  IpPacket(size_t size);

  inline
  IpPacket(uint8_t * ptr,
           size_t size);

  inline
  IpPacket(EthFrame & eth);

  inline
  uint8_t
  readIpVersion(void) const;

  inline
  void
  writeIpVersion(uint8_t ipVersion);

  inline
  uint8_t
  readIHL(void) const;

  inline
  void
  writeIHL(uint8_t ihl);

  inline
  uint8_t
  readTOS(void) const;

  inline
  void
  writeTOS(uint8_t tos);

  inline
  uint16_t
  readIpSize(void) const;

  inline
  void
  writeIpSize(uint16_t ipSize);

  inline
  uint16_t
  readIdent(void) const;

  inline
  void
  writeIdent(uint16_t ident);

  inline
  uint8_t
  readIpFlags(void) const;

  inline
  void
  writeIpFlags(uint8_t ipFlags);

  inline
  uint16_t
  readOffset(void) const;

  inline
  void
  writeOffset(uint16_t offset);

  inline
  uint8_t
  readTTL(void) const;

  inline
  void
  writeTTL(uint8_t ttl);

  inline
  uint8_t
  readIpProto(void) const;

  inline
  void
  writeIpProto(uint8_t ipProto);

  inline
  uint16_t
  readIpChecksum(void) const;

  inline
  void
  writeIpChecksum(uint16_t ipCheckSum);

  inline
  uint16_t
  computeIpChecksum(void) const;

  inline
  uint32_t
  readSrcIp(void) const;

  inline
  void
  writeSrcIp(uint32_t srcIp);

  inline
  uint32_t
  readDstIp(void) const;

  inline
  void
  writeDstIp(uint32_t dstIp);

  inline
  void
  readIpHeader(uint8_t & ipVersionOut,
               uint8_t & ihlOut,
               uint8_t & tosOut,
               uint16_t & ipSizeOut,
               uint16_t & identOut,
               uint8_t & ipFlagsOut,
               uint16_t & offsetOut,
               uint8_t & ttlOut,
               uint8_t & ipProtoOut,
               uint16_t & ipChecksumOut,
               uint32_t & srcIpOut,
               uint32_t & dstIpOut) const;

  inline
  void
  writeIpHeader(uint8_t ipVersion,
                uint8_t ihl,
                uint8_t tos,
                uint16_t ipSize,
                uint16_t ident,
                uint8_t ipFlags,
                uint16_t offset,
                uint8_t ttl,
                uint8_t ipProto,
                uint16_t ipChecksum,
                uint32_t srcIp,
                uint32_t dstIp);

  inline
  size_t
  getIpHeaderSkip(void) const;

  inline static
  uint32_t
  makeNetmask(size_t maskWidth);

  inline static
  size_t
  netmaskWidth(uint32_t netmask);

  inline static
  bool
  parseIpAddr(uint32_t & ipAddrOut,
              const char * textAddr);

  inline static
  void
  dumpIpAddr(char * textAddrOut,
             uint32_t ipAddr);

  inline static
  string
  showIpAddr(uint32_t ipAddr);

  inline
  void
  showIpHeader(ostream & output,
               size_t indent) const;

  enum
    {
    IPFLAG_RF=0x04, IPFLAG_DF=0x02, IPFLAG_MF=0x01,
    MIN_HEADER_SIZE=20,
    PROTO_ICMP=1, PROTO_TCP=6, PROTO_UDP=17, PROTO_RAWIP=255
    };

  static const uint32_t IP_GENERIC_BROADCAST=0xFFFFFFFFLU;

private:
  IpPacket(void); // no implementation
  IpPacket(const IpPacket &); // no implementation
  IpPacket & operator=(const IpPacket &); // no implementation
};

//-------- IcmpMessage -------------------------------------------------------

class IcmpMessage : public DataPacket
{
public:

  inline
  IcmpMessage(size_t size);

  inline
  IcmpMessage(uint8_t * ptr,
              size_t size);

  inline
  IcmpMessage(IpPacket & ip);

  inline
  uint8_t
  readIcmpType(void) const;

  inline
  void
  writeIcmpType(uint8_t icmpType);

  inline
  uint8_t
  readIcmpCode(void) const;

  inline
  void
  writeIcmpCode(uint8_t icmpCode);

  inline
  uint16_t
  readIcmpChecksum(void) const;

  inline
  void
  writeIcmpChecksum(uint16_t icmpChecksum);

  inline
  uint16_t
  computeIcmpChecksum(const IpPacket & insideIp) const;

  inline
  uint32_t
  readIcmpReserved(void) const;

  inline
  void
  writeIcmpReserved(uint32_t icmpReserved);

  inline
  void
  readIcmpHeader(uint8_t & icmpTypeOut,
                 uint8_t & icmpCodeOut,
                 uint16_t & icmpChecksumOut,
                 uint32_t & icmpReservedOut) const;

  inline
  void
  writeIcmpHeader(uint8_t icmpType,
                  uint8_t icmpCode,
                  uint16_t icmpChecksum,
                  uint32_t icmpReserved);

  inline
  uint16_t
  readEchoIdent(void) const;

  inline
  void
  writeEchoIdent(uint16_t echoIdent);

  inline
  uint16_t
  readEchoSeqNumber(void) const;

  inline
  void
  writeEchoSeqNumber(uint16_t echoSeqNumber);

  inline
  void
  readEchoHeader(uint8_t & icmpTypeOut,
                 uint8_t & icmpCodeOut,
                 uint16_t & icmpChecksumOut,
                 uint16_t & echoIdentOut,
                 uint16_t & echoSeqNumberOut) const;
  
  inline
  void
  writeEchoHeader(uint8_t icmpType,
                  uint8_t icmpCode,
                  uint16_t icmpChecksum,
                  uint16_t echoIdent,
                  uint16_t echoSeqNumber);

  inline
  size_t
  getIcmpHeaderSkip(void) const;

  inline
  void
  showIcmpHeader(ostream & output,
                 size_t indent) const;

  enum
    {
    HEADER_SIZE=8,
    TYPE_ECHO_REPLY=0,
    TYPE_DST_UNREACH=3,
    TYPE_SRC_QUENCH=4,
    TYPE_REDIRECT=5,
    TYPE_ECHO_REQUEST=8,
    TYPE_ROUTER_ADVERT=9,
    TYPE_ROUTER_SELECT=10,
    TYPE_TIME_EXCEED=11,
    TYPE_PARA_PROB=12,
    TYPE_TIMESTAMP_REQUEST=13,
    TYPE_TIMESTAMP_REPLY=14,
    TYPE_INFO_REQUEST=15,
    TYPE_INFO_REPLY=16,
    TYPE_ADMASK_REQUEST=17,
    TYPE_ADMASK_REPLY=18
    };

private:
  IcmpMessage(void); // no implementation
  IcmpMessage(const IcmpMessage &); // no implementation
  IcmpMessage & operator=(const IcmpMessage &); // no implementation
};

//-------- UdpDatagram -------------------------------------------------------

class UdpDatagram : public DataPacket
{
public:

  inline
  UdpDatagram(size_t size);

  inline
  UdpDatagram(uint8_t * ptr,
              size_t size);

  inline
  UdpDatagram(IpPacket & ip);

  inline
  uint16_t
  readSrcPort(void) const;

  inline
  void
  writeSrcPort(uint16_t srcPort);

  inline
  uint16_t
  readDstPort(void) const;

  inline
  void
  writeDstPort(uint16_t dstPort);

  inline
  uint16_t
  readUdpSize(void) const;

  inline
  void
  writeUdpSize(uint16_t udpSize);

  inline
  uint16_t
  readUdpChecksum(void) const;

  inline
  void
  writeUdpChecksum(uint16_t udpChecksum);

  inline
  uint16_t
  computeUdpChecksum(const IpPacket & insideIp) const;

  inline
  void
  readUdpHeader(uint16_t & srcPortOut,
                uint16_t & dstPortOut,
                uint16_t & udpSizeOut,
                uint16_t & udpChecksumOut) const;

  inline
  void
  writeUdpHeader(uint16_t srcPort,
                 uint16_t dstPort,
                 uint16_t udpSize,
                 uint16_t udpChecksum);

  inline
  size_t
  getUdpHeaderSkip(void) const;

  inline
  void
  showUdpHeader(ostream & output,
                size_t indent) const;

  enum
    {
    HEADER_SIZE=8
    };

private:
  UdpDatagram(void); // no implementation
  UdpDatagram(const UdpDatagram &); // no implementation
  UdpDatagram & operator=(const UdpDatagram &); // no implementation
};

//-------- TcpSegment --------------------------------------------------------

class TcpSegment : public DataPacket
{
public:

  inline
  TcpSegment(size_t size);

  inline
  TcpSegment(uint8_t * ptr,
             size_t size);

  inline
  TcpSegment(IpPacket & ip);

  inline
  uint16_t
  readSrcPort(void) const;

  inline
  void
  writeSrcPort(uint16_t srcPort);

  inline
  uint16_t
  readDstPort(void) const;

  inline
  void
  writeDstPort(uint16_t dstPort);

  inline
  uint32_t
  readSeqNumber(void) const;

  inline
  void
  writeSeqNumber(uint32_t seqNumber);

  inline
  uint32_t
  readAckNumber(void) const;

  inline
  void
  writeAckNumber(uint32_t ackNumber);

  inline
  uint8_t
  readTHL(void) const;

  inline
  void
  writeTHL(uint8_t thl);

  inline
  uint8_t
  readTcpFlags(void) const;

  inline
  void
  writeTcpFlags(uint8_t tcpFlags);

  inline
  uint16_t
  readWindow(void) const;

  inline
  void
  writeWindow(uint16_t window);

  inline
  uint16_t
  readTcpChecksum(void) const;

  inline
  void
  writeTcpChecksum(uint16_t tcpChecksum);

  inline
  uint16_t
  computeTcpChecksum(const IpPacket & insideIp) const;

  inline
  uint16_t
  readUrgent(void) const;

  inline
  void
  writeUrgent(uint16_t urgent);

  inline
  void
  readTcpHeader(uint16_t & srcPortOut,
                uint16_t & dstPortOut,
                uint32_t & seqNumberOut,
                uint32_t & ackNumberOut,
                uint8_t & thlOut,
                uint8_t & tcpFlagsOut,
                uint16_t & windowOut,
                uint16_t & tcpChecksumOut,
                uint16_t & urgentOut) const;

  inline
  void
  writeTcpHeader(uint16_t srcPort,
                 uint16_t dstPort,
                 uint32_t seqNumber,
                 uint32_t ackNumber,
                 uint8_t thl,
                 uint8_t tcpFlags,
                 uint16_t window,
                 uint16_t tcpChecksum,
                 uint16_t urgent);

  inline
  size_t
  getTcpHeaderSkip(void) const;

  inline
  void
  showTcpHeader(ostream & output,
                size_t indent) const;

  enum
    {
    TCPFLAG_URG=0x20, TCPFLAG_ACK=0x10, TCPFLAG_PSH=0x08,
    TCPFLAG_RST=0x04, TCPFLAG_SYN=0x02, TCPFLAG_FIN=0x01,
    MIN_HEADER_SIZE=20
    };

private:
  TcpSegment(void); // no implementation
  TcpSegment(const TcpSegment &); // no implementation
  TcpSegment & operator=(const TcpSegment &); // no implementation
};

#include "packetTypes.Ci"

#endif // PACKETTYPES_H

//----------------------------------------------------------------------------
