//----------------------------------------------------------------------------
// Copyright (c) 2007-2008 Fabrice HARROUET (ENIB)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.
// The author makes no representations about the suitability of this
// software for any purpose.
// It is provided "as is" without express or implied warranty.
//----------------------------------------------------------------------------

#ifndef SNIFFER_H
#define SNIFFER_H 1

#include "packetTypes.h"

#include <vector>

#include <string.h>
#include <errno.h>

#define RESTART_SYSCALL(result,syscall) \
        do { (result)=(syscall); } while(((result)<0)&&(errno==EINTR));

class Sniffer
{
public:

  struct Capture
    {
    uint32_t serverIp;
    uint16_t serverPort;
    uint16_t localPort;
    inline Capture(void)
           : serverIp(0), serverPort(0), localPort(0) {}
    inline Capture(uint32_t si,uint16_t sp,uint16_t lp)
           : serverIp(si), serverPort(sp), localPort(lp) {}
    };

  Sniffer(double repeatDelay,
          const string & ifName,
          uint64_t fakeMac,
          uint32_t toIp,
          const vector<uint32_t> & fromIp,
          const vector<Sniffer::Capture> & tcpCapture,
          const vector<Sniffer::Capture> & udpCapture);

  virtual
  ~Sniffer(void);

  virtual
  bool // success
  sendFrame(const EthFrame & eth);

  virtual
  bool // success
  mainLoop(void);

  virtual
  bool // allow transfert
  processPacket(EthFrame & eth,
                IpPacket & ip);

  static
  double // current time in seconds
  getTime(void);

protected:

  struct ArpInfo
    {
    uint32_t ip;
    uint64_t mac;
    double deadline;
    vector<EthFrame *> pending;
    inline ArpInfo(uint32_t i,uint64_t m)
           : ip(i),mac(m),deadline(0.0),pending() {}
    inline ~ArpInfo(void)
           {
           while(!pending.empty())
             { delete pending.back(); pending.pop_back(); }
           }
    };

  struct ClientInfo
    {
    uint32_t ip;
    uint16_t port;
    inline ClientInfo(uint32_t i,uint16_t p)
           : ip(i),port(p) {}
    };

  struct ServerInfo
    {
    uint32_t realIp;
    uint16_t realPort;
    uint16_t fakePort;
    vector<Sniffer::ClientInfo> clients;
    inline ServerInfo(uint32_t ri,uint16_t rp,uint16_t fp)
           : realIp(ri),realPort(rp),fakePort(fp),clients() {}
    };

  virtual
  double // delay in seconds
  _randomDelay(void);

  virtual
  Sniffer::ArpInfo * // arp info or 0
  _getFromArp(uint32_t ip);

  virtual
  uint64_t // mac address or 0
  _getFromMac(uint32_t ip);

  virtual
  bool // success
  _changeLocalArp(uint32_t ip,
                  uint64_t mac,
                  int flags);

  virtual
  void
  _sendArp(uint16_t operation,
           uint64_t srcMac,
           uint32_t srcIp,
           uint64_t dstMac,
           uint32_t dstIp);

  virtual
  bool // success
  _nextFrame(EthFrame & ethOut,
             double timeout);

  void
  _updateArpCache(uint32_t ip,
                  uint64_t mac);

  virtual
  bool // success
  _detectOriginalMacAddresses(double interval,
                              size_t attempts);

  static
  int // fd or -1
  _openTap(int controlFd,
           uint64_t localMac,
           size_t localMtu,
           uint32_t localIp);

  static
  int // fd or -1
  _openInterface(const string & name,
                 bool promiscuous,
                 size_t & mtuOut,
                 uint64_t & localMacOut,
                 uint32_t & localIpOut,
                 uint32_t & netMaskOut);

  static
  void
  _handler(int signum);

  static bool _stopLoop;

  double _repeatDelay;
  string _ifName;
  int _controlFd,_ethFd,_tapFd;
  size_t _mtu;
  uint8_t * _frame;
  uint64_t _fakeMac;
  uint32_t _netAddr,_netMask;
  Sniffer::ArpInfo _local,_to;
  bool _collectFrom;
  vector<Sniffer::ArpInfo> _from;
  vector<Sniffer::ServerInfo> _tcpServers;
  vector<Sniffer::ServerInfo> _udpServers;
};

#endif // SNIFFER_H

//----------------------------------------------------------------------------
